//---------------------------------------------------------------------------
#ifndef commandsH
#define commandsH

#include "typedefs.h"
#include "ethernet.h"

#define DEFAULT_RX_TIMEOUT	500 	// ms
#define DEFAULT_TX_TIMEOUT	100	// ms

#define GetCommport()		commport
#define SetCommport(x) 		commport = x

#ifdef LINUX
	#define WIN_USB_SUPPORTED	false
#else	
	#define WIN_USB_SUPPORTED	true
#endif	

enum
{
	CIN =    '\xAA',
	ESC =    '\x1B',
	ACK =	   '\x06',
	NACK =	'\x15',
	CSERR = 	'\x17',
	OVR = 	'\x19'
};

typedef enum
{
	cr_Ok,
	cr_DataError,
	cr_SendError,
	cr_ReceiveError,
	cr_Overrun,
	cr_TimeOut,
	cr_ChkSumError,
	cr_Invalid
} cmd_result_t;


typedef enum
{
	cp_serial = 0,
	cp_usb,
	cp_ethernet,
	cp_winusb
} commport_t;

extern commport_t commport;

cmd_result_t Commands_CommonSend(char *cmd, int cmd_len, int *in_len, char *in_data);
cmd_result_t Commands_Send(char *cmd, int cmd_len);

bool Commands_GetAttention(bool *was_terminal_mode);
char *Commands_GetInfo(void);
bool Commands_Reboot(void);
char *Commands_GetControllerName(void);
bool Commands_WriteAppData(char *file_name, dword cmd_baud_rate, dword upl_baud_rate);
bool Commands_PortOpen(char *serial_port, dword baud_rate, bool no_cts, commport_t cp);
bool Commands_Set_Sys_BaudRate(dword baud_rate);
bool Commands_PortClose(void);
char *Commands_GetResult(cmd_result_t cr);
dword GetMaxFlashWriteSize(void);
//---------------------------------------------------------------------------
#endif

