#ifndef errorsH
#define errorsH

#include "typedefs.h"

typedef struct
{
	int err_no;
	char *errmsg;
} err_struct;

extern err_struct errors[];

#define HAS_SYS_ERRNO	(0x40 << 8)
#define HAS_WSA_ERRNO	(0x20 << 8)
#define SERIAL_ERR 		(1 << 8)
#define CMD_ERR 			(2 << 8)
#define USB_ERR			(3 << 8)
#define ETH_ERR			(4 << 8)

#define SER_ERR_TOUT					   SERIAL_ERR + 1 + HAS_SYS_ERRNO
#define SER_ERR_TOUT_NOT_OPEN		   SERIAL_ERR + 2
#define SER_ERR_CONFIG				   SERIAL_ERR + 3 + HAS_SYS_ERRNO
#define SER_ERR_CONFIG_NOT_OPEN	   SERIAL_ERR + 4
#define SER_ERR_COMSTATE			   SERIAL_ERR + 5 + HAS_SYS_ERRNO
#define SER_ERR_ALREADY_OPEN		   SERIAL_ERR + 6
#define SER_ERR_CANT_OPEN			   SERIAL_ERR + 7 + HAS_SYS_ERRNO
#define SER_ERR_OPEN_NO_CHAR_DEV	   SERIAL_ERR + 8
#define SER_ERR_CLOSE_NOT_OPEN	   SERIAL_ERR + 9
#define SER_ERR_CANT_CLOSE			   SERIAL_ERR + 10
#define SER_ERR_TX_NOT_OPEN         SERIAL_ERR + 11
#define SER_ERR_TX					   SERIAL_ERR + 12 + HAS_SYS_ERRNO
#define SER_ERR_TX_SIZE				   SERIAL_ERR + 13
#define SER_ERR_TX_TIMEOUT		SERIAL_ERR + 14
#define SER_ERR_RX_NOT_OPEN		   SERIAL_ERR + 15
#define SER_ERR_NOT_SERIAL				SERIAL_ERR + 16
#define SER_ERR_RX					   SERIAL_ERR + 17 + HAS_SYS_ERRNO
#define SER_ERR_RX_NO_DATA			   SERIAL_ERR + 18
#define SER_INVALID_NAME				SERIAL_ERR + 19

#define CMD_ERR_CANT_ERASE	   		CMD_ERR + 1
#define CMD_ERR_CANT_SET_SUB_PAGE	CMD_ERR + 2
#define CMD_ERR_CANT_ERASE_IDX		CMD_ERR + 3
#define CMD_ERR_NSUP_SW_BAUD_RATE	CMD_ERR + 4
#define CMD_ERR_CANT_SW_BAUD_RATE   CMD_ERR + 5
#define CMD_ERR_HAS_UPLD_BAUD_RATE  CMD_ERR + 6
#define CMD_ERR_CANT_WRITE	   		CMD_ERR + 7
#define CMD_ERR_CANT_READ_FILE  		CMD_ERR + 8
#define CMD_ERR_REACHED_EOF			CMD_ERR + 9
#define CMD_ERR_CANT_OPEN_FILE		CMD_ERR + 10
#define CMD_ERR_FILESIZE				CMD_ERR + 11
#define CMD_ERR_NOT_ILCD				CMD_ERR + 12
#define CMD_ERR_ALREADY_OPEN			CMD_ERR + 13
#define CMD_ERR_ALREADY_CLOSED		CMD_ERR + 14
#define CMD_ERR_PORT_NOT_OPEN			CMD_ERR + 15
#define CMD_ERR_FILE_TOO_BIG        CMD_ERR + 16
#define CMD_ERR_CANT_GET_CNAME		CMD_ERR + 17
#define CMD_ERR_INVAL_CHKSUM			CMD_ERR + 18
#define CMD_ERR_IVAL_CONTR				CMD_ERR + 19
#define CMD_ERR_USER_ABORT				CMD_ERR + 20

#define USB_ERR_DLL_LOAD				USB_ERR + 1
#define USB_ERR_DLL_ENTRY_POINT		USB_ERR + 2
#define USB_ERR_ENUM						USB_ERR + 3 + HAS_SYS_ERRNO
#define USB_ERR_ALREADY_OPEN			USB_ERR + 4
#define USB_ERR_WR_CREATE_FILE		USB_ERR + 5 + HAS_SYS_ERRNO
#define USB_ERR_NOT_OPEN				USB_ERR + 6
#define USB_ERR_WRITE_TIMEOUT       USB_ERR + 7 + HAS_SYS_ERRNO
#define USB_ERR_WRITE					USB_ERR + 8 + HAS_SYS_ERRNO
#define USB_ERR_PARTIAL_WRITE       USB_ERR + 9
#define USB_ERR_CONNECTION_LOST     USB_ERR + 10 + HAS_SYS_ERRNO
#define USB_ERR_RX_NO_DATA				USB_ERR + 11
#define USB_ERR_ENUM_DEV_SET			USB_ERR + 12
#define USB_ERR_NO_DEVICES_FOUND    USB_ERR + 13
#define USB_ERR_LIBUSB_INIT    		USB_ERR + 14
#define USB_RD_THREAD_CREATE_ERROR	USB_ERR + 15
#define USB_RD_CREATE_FILE_ERR		USB_ERR + 16 + HAS_SYS_ERRNO

#define ETH_ERR_WSA_LOAD           	ETH_ERR + 1  + HAS_WSA_ERRNO
#define ETH_ERR_CREATE_UDP   			ETH_ERR + 2  + HAS_WSA_ERRNO
#define ETH_ERR_CREATE_TCP   			ETH_ERR + 3  + HAS_WSA_ERRNO
#define ETH_ERR_SET_SOCK_OPTION     ETH_ERR + 4  + HAS_WSA_ERRNO
#define ETH_ERR_SEND_UDP_BROADCAST  ETH_ERR + 5  + HAS_WSA_ERRNO
#define ETH_ERR_UDP_BIND            ETH_ERR + 6  + HAS_WSA_ERRNO
#define ETH_ERR_UDP_RESPONSE  		ETH_ERR + 7
#define ETH_ERR_RECEIVE_TIMEOUT     ETH_ERR + 8
#define ETH_ERR_WRONG_PW            ETH_ERR + 9
#define ETH_ERR_RECEIVE					ETH_ERR + 10
#define ETH_ERR_NO_DEVICES_FOUND		ETH_ERR + 11
#define ETH_ERR_TCP_CONNECT			ETH_ERR + 12 + HAS_WSA_ERRNO
#define ETH_ERR_NOT_CONNECTED			ETH_ERR + 13
#define ETH_ERR_MORE_DEVICES_FOUND	ETH_ERR + 14
#define ETH_ERR_NO_PW					ETH_ERR + 15
#define ETH_ALREADY_CONNECTED			ETH_ERR + 16

extern bool Error_SetErr(int error_no);
extern int Error_GetLastErrNo(void);
extern char *Error_GetLastErrMsg(void);
extern int Error_GetLastSysErrNo(void);

#endif
